<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class VendasRealizadasPeriodo {

	public function __construct(EntityManager $entityManager, Utilities\Util $utilService) {
        ini_set('max_execution_time', 1800);
        ini_set('memory_limit', '1G');
        $this->entityManager = $entityManager;
        $this->utilService = $utilService;
    }

    public function getReport($conditions) {
    	$arrVendasPeriodo = [];
	    $arrTipoRecebimento = [];
	    $arrVendasPeriodo = $this->entityManager->getConnection()->fetchAll(self::REL_VENDAS_REALIZADAS_PERIODO, $conditions);
	
    	$params = [
    		":CDFILIAL" => $conditions['CDFILIAL'],
    		":DTINI" => $conditions['DTINI'],
    		":DTFIM" => $conditions['DTFIM'],
		];
	    
	    foreach($arrVendasPeriodo as $chVP => $valVP){
	        $params += [
    			":CDFILICAIXA" => $valVP['CDFILICAIXA'],
    			":CDFILICLIENTE" => $valVP['CDFILICLIENTE'],
    			":CDCONSCLI" => $valVP['CDCONSCLI'],    
    			":NRSEQVENDA" => $valVP['NRSEQVENDA'],
    			":CUPOM" => $valVP['CUPOM']
    		];
    		
    		$arrTipoRecebimento = $this->entityManager->getConnection()->fetchAll(self::TIPOS_RECEBIMENTOS_REALIZADAS_PERIODO, $params);
    		foreach($arrTipoRecebimento as $chTR => $valTR){
    			$arrVendasPeriodo[$chVP]["LABELVENDARECEBIMENTO"] .= " | " . $arrTipoRecebimento[$chTR]["NMTIPORECE"] . ": R$". $this->utilService->formataValor($arrTipoRecebimento[$chTR]["VRMOVIVEND"],2,",",".");   
    		}
    		
    		unset($params[":CDFILICAIXA"]);
    		unset($params[":CDFILICLIENTE"]);
    		unset($params[":CDCONSCLI"]);
    		unset($params[":NRSEQVENDA"]);
    		unset($params[":CUPOM"]);
    		
    	}
    	unset($params);
    	unset($arrTipoRecebimento);
	    return $arrVendasPeriodo;
	}

    const REL_VENDAS_REALIZADAS_PERIODO = "
        SELECT B.*, (CASE 
                        WHEN PV.IDENVIACDBARXML = 'X' AND PX.CDPRODUTOEXT IS NOT NULL
                            THEN PX.CDPRODUTOEXT 
                        ELSE 'Sem Código externo' 
                    END) AS CDPRODUTOEXT
            FROM (
                SELECT F.CDFILIAL || ' - ' || F.NMFILIAL AS LABELFILIAL, F.CDFILIAL,
                   'Caixa : ' || V.CDCAIXA || ' | Nr. Ven.: ' || V.NRSEQVENDA || ' | Dt. Ven.: ' || TO_CHAR(V.DTVENDA, 'DD/MM/YYYY HH24:MI:SS') || ' | Cup./N. Fiscal: ' || NVL(V.NRCUPOMFIS, V.NRNOTAFISCALCE) || ' | Cli./Cons.: ' || C.NMCONSUMIDOR || ' | Vend.: ' || VE.NMRAZSOCVEN || ' | Valor Repique: ' || NVL(V.VRREPIQUEVENDA, 0) || ' | Modalidade: ' || NVL(O.DSORIGEMVENDA, 'Modalidade Desconhecida') AS LABELVENDARECEBIMENTO, 
                        P.CDARVPROD, P.CDPRODUTO, P.NMPRODUTO, P.SGUNIDADE, (F.CDFILIAL||V.CDCAIXA) AS CDFILICAIXA, V.NRSEQVENDA, NVL(V.NRCUPOMFIS, V.NRNOTAFISCALCE) AS CUPOM, (V.CDFILIAL||V.CDCLIENTE) AS CDFILICLIENTE, (V.CDCLIENTE || V.CDCONSUMIDOR) AS CDCONSCLI,
                        IT.VRUNITVEND, IT.QTPRODVEND AS QTPRODVEND, 
                        IT.VRDESITVEND, IT.VRACRITVEND,   
                        TRUNC( IT.QTPRODVEND * ( IT.VRUNITVEND + NVL( IT.VRUNITVENDCL, 0)), 2) - NVL(IT.VRDESITVEND, 0) AS VRBRUTO,
                        TRUNC( IT.QTPRODVEND * ( IT.VRUNITVEND + NVL( IT.VRUNITVENDCL, 0)), 2) - (NVL(IT.VRDESITVEND, 0)) + NVL(IT.VRACRITVEND, 0)  AS VRTOTAL
                    FROM VENDA V
                        JOIN ITEMVENDA IT
                            ON V.CDFILIAL = IT.CDFILIAL 
                                AND V.CDCAIXA    = IT.CDCAIXA
                                AND V.NRSEQVENDA = IT.NRSEQVENDA
                        JOIN PRODUTO P
                            ON P.CDPRODUTO = IT.CDPRODUTO 
                        LEFT OUTER JOIN VENDEDOR VE
                            ON V.CDVENDEDOR = VE.CDVENDEDOR
                        JOIN (
                            SELECT CDFILIAL, NMFILIAL, CDCAIXA
                                FROM V_FILIALLOJACAIXA FLC 
                                WHERE (INSTR( :CDFILIAL, FLC.CDFILIAL) <> 0)
                                    AND (((INSTR(:CDFILICAIXA, FLC.CDFILIAL || FLC.CDCAIXA) <> 0)) OR (:TCDFILICAIXA = 'T'))
                        ) F
                            ON V.CDFILIAL = F.CDFILIAL
                            AND V.CDCAIXA = F.CDCAIXA
                        LEFT OUTER JOIN (
                            SELECT CL.CDCLIENTE, CL.NMRAZSOCCLIE, CO.CDCONSUMIDOR, CO.NMCONSUMIDOR
                                FROM CONSUMIDOR CO, CLIENTE CL
                                WHERE CO.CDCLIENTE = CL.CDCLIENTE 
                        ) C
                            ON V.CDCLIENTE = C.CDCLIENTE
                                AND V.CDCONSUMIDOR = C.CDCONSUMIDOR
                        LEFT JOIN ORIGEMVENDAFOS O
                            ON V.IDORIGEMVENDA = O.IDORIGEMVENDA
                                AND V.NRORG = O.NRORG
                    WHERE V.DTENTRVENDA BETWEEN :DTINI AND :DTFIM
                        AND (((INSTR(:IDSITUVENDA, V.IDSITUVENDA) <> 0)) OR (:IDSITUVENDA = 'T'))
                        AND ((INSTR( :CDFILICLIENTE, V.CDFILIAL||V.CDCLIENTE) <> 0) OR (:TCDFILICLIENTE = 'T'))
                        AND (((INSTR(:CDCONSCLI, C.CDCLIENTE || V.CDCONSUMIDOR) <> 0)) OR (:TCDCONSCLI = 'T'))
                ORDER BY F.CDFILIAL, V.CDCAIXA, V.DTVENDA, P.CDARVPROD
            ) B 
            JOIN PARAVEND PV 
                ON PV.CDFILIAL = B.CDFILIAL 
            LEFT JOIN PRODEXTERNO PX 
                ON B.CDFILIAL = PX.CDFILIAL 
                    AND B.CDPRODUTO = PX.CDPRODUTO
    ";
    const TIPOS_RECEBIMENTOS_REALIZADAS_PERIODO = "
	SELECT E.CDTIPORECE, T.NMTIPORECE, E.CDFILIAL, E.CDCAIXA, E.NRSEQVENDA, E.CUPOM, (E.VRMOVIVEND - NVL(S.VRMOVIVEND,0)) AS VRMOVIVEND
		FROM TIPORECE T, (SELECT SUM(M.VRMOVIVEND) AS VRMOVIVEND, M.CDFILIAL, M.CDCAIXA, M.NRSEQVENDA, M.CDTIPORECE, VD.CUPOM
							FROM MOVCAIXA M
							JOIN (SELECT V.CDFILIAL, V.NRSEQVENDA, NVL(V.NRCUPOMFIS, V.NRNOTAFISCALCE) AS CUPOM,
										V.CDCAIXA
									FROM VENDA V
									JOIN (SELECT FLC.CDCAIXA, FLC.NMCAIXA, FLC.NMFILIAL, FLC.CDFILIAL, FLC.CDLOJA, FLC.NMLOJA
											FROM V_FILIALLOJACAIXA FLC
										WHERE ((INSTR(:CDFILIAL, FLC.CDFILIAL ) <> 0) OR (:CDFILIAL = 'T')) 
											AND (FLC.CDFILIAL||FLC.CDCAIXA) = :CDFILICAIXA   
										)FILOCA 
									ON V.CDFILIAL = FILOCA.CDFILIAL
									AND V.CDCAIXA = FILOCA.CDCAIXA
									AND V.CDLOJA = FILOCA.CDLOJA
								WHERE (V.CDFILIAL||V.CDCLIENTE) = :CDFILICLIENTE 
									AND (V.CDCLIENTE || V.CDCONSUMIDOR) = :CDCONSCLI
									AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIM) VD
							ON M.CDFILIAL = VD.CDFILIAL
							AND M.CDCAIXA = VD.CDCAIXA
							AND M.NRSEQVENDA = VD.NRSEQVENDA
							AND (VD.NRSEQVENDA = :NRSEQVENDA) 
							AND VD.CUPOM = :CUPOM
						WHERE M.IDTIPOMOVIVE IN ('E' , 'T')
						GROUP BY M.CDFILIAL, M.CDCAIXA, M.NRSEQVENDA, M.CDTIPORECE, VD.CUPOM) E,
			(SELECT SUM(M.VRMOVIVEND) AS VRMOVIVEND, M.CDFILIAL, M.CDCAIXA, M.NRSEQVENDA, M.CDTIPORECE, VD.CUPOM
				FROM MOVCAIXA M
				JOIN (SELECT V.CDFILIAL, V.NRSEQVENDA, NVL(V.NRCUPOMFIS, V.NRNOTAFISCALCE) AS CUPOM,
							V.CDCAIXA
						FROM VENDA V
						JOIN (SELECT FLC.CDCAIXA, FLC.NMCAIXA, FLC.NMFILIAL, FLC.CDFILIAL, FLC.CDLOJA, FLC.NMLOJA
								FROM V_FILIALLOJACAIXA FLC
							WHERE ((INSTR(:CDFILIAL, FLC.CDFILIAL ) <> 0) OR (:CDFILIAL = 'T')) 
								AND (FLC.CDFILIAL||FLC.CDCAIXA) = :CDFILICAIXA    
							)FILOCA 
						ON V.CDFILIAL = FILOCA.CDFILIAL
						AND V.CDCAIXA = FILOCA.CDCAIXA
						AND V.CDLOJA = FILOCA.CDLOJA
					WHERE (V.CDFILIAL||V.CDCLIENTE) = :CDFILICLIENTE 
						AND (V.CDCLIENTE || V.CDCONSUMIDOR) = :CDCONSCLI
						AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIM) VD
				ON M.CDFILIAL = VD.CDFILIAL
				AND M.CDCAIXA = VD.CDCAIXA
				AND M.NRSEQVENDA = VD.NRSEQVENDA
				AND (VD.NRSEQVENDA = :NRSEQVENDA) 
				AND VD.CUPOM = :CUPOM
			WHERE M.IDTIPOMOVIVE = 'S'
			GROUP BY M.CDFILIAL, M.CDCAIXA, M.NRSEQVENDA, M.CDTIPORECE, VD.CUPOM) S
	WHERE E.CDFILIAL = S.CDFILIAL (+)
		AND E.CDCAIXA = S.CDCAIXA (+)
		AND E.NRSEQVENDA = S.NRSEQVENDA (+)
		AND E.CDTIPORECE = S.CDTIPORECE (+)
		AND E.CDTIPORECE = T.CDTIPORECE 
		AND T.IDTIPORECE NOT IN ('5','7')

	UNION

	SELECT E.CDTIPORECE, T.NMTIPORECE, E.CDFILIAL, E.CDCAIXA, E.NRSEQVENDA, E.CUPOM, (E.VRMOVIVEND - NVL(S.VRMOVIVEND,0)) AS VRMOVIVEND
			FROM TIPORECE T, (SELECT SUM(M.VRMOVIVEND) AS VRMOVIVEND, M.CDFILIAL, M.CDCAIXA, M.NRSEQVENDA, M.CDTIPORECE, VD.CUPOM
								FROM MOVCAIXA M
								JOIN (SELECT V.CDFILIAL, V.NRSEQVENDA, NVL(V.NRCUPOMFIS, V.NRNOTAFISCALCE) AS CUPOM,
											V.CDCAIXA
										FROM VENDA V
										JOIN (SELECT FLC.CDCAIXA, FLC.NMCAIXA, FLC.NMFILIAL, FLC.CDFILIAL, FLC.CDLOJA, FLC.NMLOJA
												FROM V_FILIALLOJACAIXA FLC
											WHERE ((INSTR(:CDFILIAL, FLC.CDFILIAL ) <> 0) OR (:CDFILIAL = 'T')) 
												AND (FLC.CDFILIAL||FLC.CDCAIXA) = :CDFILICAIXA    
											)FILOCA 
										ON V.CDFILIAL = FILOCA.CDFILIAL
										AND V.CDCAIXA = FILOCA.CDCAIXA
										AND V.CDLOJA = FILOCA.CDLOJA
									WHERE (V.CDFILIAL||V.CDCLIENTE) = :CDFILICLIENTE 
										AND (V.CDCLIENTE || V.CDCONSUMIDOR) = :CDCONSCLI
										AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIM) VD
								ON M.CDFILIAL = VD.CDFILIAL
								AND M.CDCAIXA = VD.CDCAIXA
								AND M.NRSEQVENDA = VD.NRSEQVENDA
								AND (VD.NRSEQVENDA = :NRSEQVENDA) 
								AND VD.CUPOM = :CUPOM
							WHERE M.IDTIPOMOVIVE IN ('E' , 'T')
							GROUP BY M.CDFILIAL, M.CDCAIXA, M.NRSEQVENDA, M.CDTIPORECE, VD.CUPOM) E,
				(SELECT SUM(M.VRMOVIVEND) AS VRMOVIVEND, M.CDFILIAL, M.CDCAIXA, M.NRSEQVENDA, M.CDTIPORECE, VD.CUPOM
					FROM MOVCAIXA M
					JOIN (SELECT V.CDFILIAL, V.NRSEQVENDA, NVL(V.NRCUPOMFIS, V.NRNOTAFISCALCE) AS CUPOM,
								V.CDCAIXA
							FROM VENDA V
							JOIN (SELECT FLC.CDCAIXA, FLC.NMCAIXA, FLC.NMFILIAL, FLC.CDFILIAL, FLC.CDLOJA, FLC.NMLOJA
									FROM V_FILIALLOJACAIXA FLC
								WHERE ((INSTR(:CDFILIAL, FLC.CDFILIAL ) <> 0) OR (:CDFILIAL = 'T')) 
									AND (FLC.CDFILIAL||FLC.CDCAIXA) = :CDFILICAIXA    
								)FILOCA 
							ON V.CDFILIAL = FILOCA.CDFILIAL
							AND V.CDCAIXA = FILOCA.CDCAIXA
							AND V.CDLOJA = FILOCA.CDLOJA
						WHERE (V.CDFILIAL||V.CDCLIENTE) = :CDFILICLIENTE 
							AND (V.CDCLIENTE || V.CDCONSUMIDOR) = :CDCONSCLI
							AND V.DTENTRVENDA BETWEEN :DTINI AND :DTFIM) VD
					ON M.CDFILIAL = VD.CDFILIAL
					AND M.CDCAIXA = VD.CDCAIXA
					AND M.NRSEQVENDA = VD.NRSEQVENDA
					AND (VD.NRSEQVENDA = :NRSEQVENDA) 
					AND VD.CUPOM = :CUPOM
				WHERE M.IDTIPOMOVIVE = 'S'
				GROUP BY M.CDFILIAL, M.CDCAIXA, M.NRSEQVENDA, M.CDTIPORECE, VD.CUPOM) S
		WHERE E.CDFILIAL = S.CDFILIAL (+)
			AND E.CDCAIXA = S.CDCAIXA (+)
			AND E.NRSEQVENDA = S.NRSEQVENDA (+)
			AND E.CDTIPORECE = T.CDTIPORECE
			AND T.IDTIPORECE IN ('5','7')
	";
}